# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django import forms
from django.core.exceptions import ValidationError
from django.utils import simplejson
from project.view import FuncView, RootView, MenuView
import os
import re
from project.fal.fxwf import Fxwf as Cloud
from project.forms.fields.extras.fxwf import JobListCheckboxField
from project.forms.fields.extras.fxwf import SidHiddenField
from project.forms.fields.extras.fxwf import SharenameSelectField
from project.forms.fields.extras.fxwf import SharenameHiddenField
from project.forms.fields.extras.fxwf import EnableField
from project.forms.fields.extras.fxwf import SyncDirectionRadioField
from project.forms.fields.extras.fxwf import IntervalSelectField
from project.forms.fields.extras.fxwf import UploadsizeTextField
from project.forms.fields.extras.fxwf import FXWFUidTextField
from project.forms.fields.extras.fxwf import FXWFCabinetTextField
from project.forms.fields.extras.fxwf import FXWFPasswordField
from project.forms.fields.extras.fxwf import FXWFDrawerTextField
from project.forms.fields.checkbox import CheckboxField


MAX_JOB_NUMBER = 10


class MenuView(MenuView):
    pass


class ListForm(forms.Form):

    def __init__(self, choices=[], *args, **kwargs):
        super(ListForm, self).__init__(*args, **kwargs)
        self.fields["joblist"] = JobListCheckboxField(
                label=_(u"FXWF: Target Config List"),
                choices=choices,
                required=False)


class ListView(FuncView):

    views = ["delete", "download"]

    def view(self, request):
        joblist, form = self.__get_joblist_and_form()

        template = os.path.join(self.TEMPLATES, "list.html")
        dictionary = {
            "form": form,
            "joblist": joblist,
            "overmax": (len(Cloud().load()) >= MAX_JOB_NUMBER),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_delete(self, request):
        joblist, form = self.__get_joblist_and_form(data=request.POST)

        with self.processing() as p:
            if form.is_valid():
                cloud = Cloud()
                p.result(True)
                if len(form.cleaned_data["joblist"]):
                    for sid in form.cleaned_data["joblist"]:
                        if not cloud.delete(sid):
                            p.result(False)
                            self.set_message(cloud.get_message())
                    if p.result():
                        self.set_message(cloud.get_message())
                else:
                    p.result(False)
                    self.set_message(_(u"FXWF: TargetCount(0)"))
        # NOTE 削除により一覧に変更があるはずなので再生成
        joblist, form = self.__get_joblist_and_form()

        template = os.path.join(self.TEMPLATES, "list.html")
        dictionary = {
            "form": form,
            "joblist": joblist,
            "overmax": (len(Cloud().load()) >= MAX_JOB_NUMBER),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_download(self, request):
        joblist, form = self.__get_joblist_and_form(data=request.POST)

        with self.processing() as p:
            if form.is_valid():
                cloud = Cloud()
                p.result(True)
                if len(form.cleaned_data["joblist"]):
                    for sid in form.cleaned_data["joblist"]:
                        if not cloud.download(sid):
                            p.result(False)
                            sharename = cloud.query(sid).get("share", {}).get("name", "-")
                            self.set_message(_(u"FXWF: Failed to start download. (%(sharename)s)") % {"sharename": sharename})
                    if p.result():
                        self.set_message(_(u"FXWF: Successed to start download."))
                else:
                    p.result(False)
                    self.set_message(_(u"CHBox2: TargetCount(0)"))

        template = os.path.join(self.TEMPLATES, "list.html")
        dictionary = {
            "form": form,
            "joblist": joblist,
            "overmax": (len(Cloud().load()) >= MAX_JOB_NUMBER),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __get_joblist(self):
        joblist = []
        for job in Cloud().load().values():
            try:
                data = Cloud().query(job["sid"])
                joblist.append({
                    "sid": data["sid"],
                    "enable": data.get("enable"),
                    "srcname": data.get("share", {}).get("name"),
                    "status": {
                        "noerror": "o",
                        "fail": "x",
                    }.get(data["status"], "x") if data["enable"] else "-",
                    "interval": data.get("downloadinterval"),
                    "dstname": data.get("fxwf", {}).get("drawer", {}).get("name", u""),
                    "dstdesc": data.get("fxwf", {}).get("drawer", {}).get("desc", u""),
                })
            except:
                pass
        return joblist

    def __get_joblist_and_form(self, data=None):
        try:
            joblist = self.__get_joblist()
            form = ListForm(
                    data=data,  # request.POST
                    choices=[(j["sid"], j["srcname"]) for j in joblist])
            for job, cb in zip(joblist, form["joblist"]):
                job["cb"] = cb
            return joblist, form
        except:
            return [], ListForm(data=data, choices=[])


class JobForm(forms.Form):

    def __init__(self, mode="add", *args, **kwargs):
        super(JobForm, self).__init__(*args, **kwargs)
        self.fields["sid"] = SidHiddenField(required=(mode == "modify"))
        self.fields["sharename"] = (SharenameHiddenField if mode == "modify" else SharenameSelectField)(
                label=_(u"FXWF: Target Share Name"),
                required=True)

    enable = EnableField(
            label=_(u"FXWF: Working Folder Sync"),
            initial=True,
            required=True)
    _(u"FXWF: Bidirectionally")
    _(u"FXWF: Upload Only")
    _(u"%(h)d hours") % {"h": 1}
    download = SyncDirectionRadioField(
            label=_(u"FXWF: Sync Direction"),
            required=True)
    interval = IntervalSelectField(
            label=_(u"FXWF: Monitoring Interval"),
            required=False)
    uploadsize = UploadsizeTextField(
            label=_(u"FXWF: Max Upload File Size"),
            required=False)
    uid = FXWFUidTextField(
            label=_(u"FXWF: User ID"),
            required=False)
    passwd = FXWFPasswordField(
            label=_(u"FXWF: Password"),
            required=False)
    cabinet = FXWFCabinetTextField(
            label=_(u"FXWF: Cabinet"),
            required=False)
    drawer = FXWFDrawerTextField(
            label=_(u"FXWF: Drawer"),
            required=False)

    def clean(self):
        data = self.cleaned_data
        if data.get("enable", False):
            if data.get("uid", None) in [""]:
                raise ValidationError("%s: %s" % (
                        _(u"FXWF: User ID"),
                        _(u"This field is required.")))
            if data.get("passwd", None) in [""]:
                raise ValidationError("%s: %s" % (
                        _(u"FXWF: Password"),
                        _(u"This field is required.")))
            if data.get("cabinet", None) in [""]:
                raise ValidationError("%s: %s" % (
                        _(u"FXWF: Cabinet"),
                        _(u"This field is required.")))
            if data.get("drawer", None) in [""]:
                raise ValidationError("%s: %s" % (
                        _(u"FXWF: Drawer"),
                        _(u"This field is required.")))
        return data


class JobView(FuncView):

    mode = None

    views = ['apply']

    def __initial(self, sid):
        if sid in [None, "", [], ()]:
            return {}
        try:
            job = Cloud().query(sid)
            return {
                "sid": job["sid"],
                "sharename": job["share"]["name"],
                "enable": job.get("enable", False),
                "uid": job.get("fxwf", {}).get("uid", ""),
                "passwd": job.get("fxwf", {}).get("passwd", ""),
                "cabinet": job.get("fxwf", {}).get("cabinet", ""),
                "drawer": job.get("fxwf", {}).get("drawer", {}).get("name", ""),
                "download": int(job.get("downloadinterval", 0)) > 0,
                "interval": job.get("downloadinterval", 0),
                "uploadsize": job.get("uploadmaxfilesize", 0),
            }
        except:
            return {}

    def view(self, request, sid=None):
        form = JobForm(initial=self.__initial(sid), mode=self.mode)

        template = os.path.join(self.TEMPLATES, "job.html")
        dictionary = {
            "direction_is_readonly": not Cloud().dlmode(),
            "mode": self.mode,
            "overmax": (len(Cloud().load()) >= MAX_JOB_NUMBER),
            "form": form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request, sid=None):
        form = JobForm(data=request.POST, mode=self.mode)

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                cloud = Cloud()
                desc = None if self.mode == "modify" else ""
                if form.cleaned_data["enable"]:
                    try:
                        input_name = form.cleaned_data["drawer"].encode("utf-8")
                    except:
                        input_name = form.cleaned_data["drawer"]
                    for drawer in Cloud().list(
                            uid=form.cleaned_data["uid"],
                            passwd=form.cleaned_data["passwd"],
                            cabinet=form.cleaned_data["cabinet"]):
                        if drawer["name"] == input_name:
                            desc = drawer["desc"]
                            break
                job = {
                    "sid": form.cleaned_data["sid"],
                    "enable": form.cleaned_data["enable"],
                    #"downloadinterval": form.cleaned_data["interval"],
                    "downloadinterval": form.cleaned_data["interval"] if form.cleaned_data["download"] else 0,
                    "uploadmaxfilesize": form.cleaned_data["uploadsize"] or 0,
                    "share": {
                        "name": form.cleaned_data["sharename"],
                    },
                    "fxwf": {
                        "uid": form.cleaned_data["uid"],
                        "passwd": form.cleaned_data["passwd"],
                        "cabinet": form.cleaned_data["cabinet"],
                        "drawer": {
                            "name": form.cleaned_data["drawer"],
                            "desc": desc,
                        },
                    },
                }
                p.result((cloud.modify if self.mode == "modify" else cloud.add)(job))
                self.set_message(cloud.get_message())
                if p.result():
                    # sharenameのchoicesが変わるかもしれないため
                    form = JobForm(initial=request.POST, mode=self.mode)

        template = os.path.join(self.TEMPLATES, "job.html")
        dictionary = {
            "direction_is_readonly": not Cloud().dlmode(),
            "mode": self.mode,
            "overmax": (len(Cloud().load()) >= MAX_JOB_NUMBER),
            "form": form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }


class AddView(JobView):

    mode = "add"


class ModifyView(JobView):

    mode = "modify"


class DataListView(RootView):

    MAX = 10

    def view(self, request):
        datalist = []
        try:
            cleaned_data = self.__cleaned_data(request.POST)
            if cleaned_data is None:
                raise
            cloud = Cloud()
            for drawer in cloud.list(
                    uid=cleaned_data["uid"],
                    passwd=cleaned_data["passwd"],
                    cabinet=cleaned_data["cabinet"]):
                try:
                    if re.search(
                                r"%s" % cleaned_data["drawer"],
                                unicode(drawer["name"], 'utf-8')):
                        datalist.append({
                            "label": [drawer["name"], drawer["desc"]],
                            "value": drawer["name"],
                        })
                except:
                    pass
            #datalist.sort(key=lambda data: data["label"])
            #datalist = datalist[:self.MAX]
            from fal_info import Results
            result = int(cloud.get_result())
            if cleaned_data["drawer"] == "":
                if result == int(Results.OK):
                    if len(datalist) == 0:
                        datalist.insert(0, {
                            "label": [_(u"FXWF: Not Found (Drawer)"), ""],
                            "value": "",
                        })
                else:
                    datalist.insert(0, {
                        "label": [_(u"FXWF: Failed to get drawers."), ""],
                        "value": "",
                    })
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(
                    simplejson.dumps(datalist),
                    mimetype="application/json;charset=UTF-8;"),
        }

    def __cleaned_data(self, data):
        try:
            form = JobForm()
            cleaned_data = {}
            for name in ["uid", "passwd", "cabinet", "drawer"]:
                cleaned_data[name] = form.fields[name].clean(data[name])
            return cleaned_data
        except:
            return None


class DLModeForm(forms.Form):

    #dlmode = CheckboxField(
    #        label=_(u"FXWF: DLMode"),
    #        required=False)
    dlmode = EnableField(
            label=_(u"FXWF: DLMode"),
            required=False)


class DLModeView(FuncView):

    views = ['apply']

    def view(self, request):
        form = DLModeForm(initial={'dlmode': Cloud().dlmode()})

        template = os.path.join(self.TEMPLATES, "dlmode.html")
        dictionary = {
            "form": form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        form = DLModeForm(data=request.POST)

        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
                cloud = Cloud()
                dlmode = cloud.dlmode(enable=form.cleaned_data['dlmode'])
                if isinstance(dlmode, bool):
                    p.result(True)
                self.set_message(cloud.get_message())

        template = os.path.join(self.TEMPLATES, "dlmode.html")
        dictionary = {
            "form": form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }
